# Copyright 2023 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

import logging
from datetime import datetime

from odoo import _, models
from odoo.exceptions import UserError

_logger = logging.getLogger(__name__)


class ApiCashpad(models.AbstractModel):
    _name = "api.cashpad"
    _inherit = ["api.connector"]
    _description = "API Cashpad"
    _backend_name = "backend.cashpad"

    # ------------------------------------------------------
    # Common functions
    # ------------------------------------------------------
    def _get_cashpad(self, url, params=None, backend_id=False, all_response=False):
        """
        Fonction d'appel à Cashpad

        :param str url: URL du endpoint
        :param dict params: paramètres de l'appel (optionnel)
        :param obj backend_id: backend spécifique, objet backend.cashpad (optionnel)
        :param bool all_response: réponse globale, pas que les données (optionnel)

        :returns dict: réponse de l'API
        """
        backend = self._get_backend() if not backend_id else backend_id
        params.update(
            {
                "apiuser_email": backend.login,
                "apiuser_token": backend.password,
            }
        )
        try:
            response = self.api_get(
                url=backend.url + url,
                verify=backend.ssl,
                timeout=backend.timeout,
                params=params,
            )
            if response.get("error"):
                raise UserError(_("API Error: \n%s") % (response.get("error"),))
            if all_response:
                return response
            else:
                return response.get("data")
        except Exception as e:
            raise e

    # ------------------------------------------------------
    # Sales retrieval
    # ------------------------------------------------------
    def _get_archive_list(
        self,
        installation_id,
        start_sequential_id=None,
        backend_id=False,
        all_response=False,
    ):
        """
        Retourne la liste des archives à partir d'une séquence initiale
        https://cashpad.notion.site/Archives-listing-cc40943a81184a189e9b84130c9306a6

        :param integer installation_id: id de la caisse
        :param integer start_sequential_id: id de la séquence minimale (optionnel)
        :param bool all_response: réponse globale, pas que les données (optionnel)

        :returns dict: réponse de l'API
        """
        url = "/api/salesdata/v2/" + installation_id + "/archives"
        params = (
            {"start_sequential_id": start_sequential_id} if start_sequential_id else {}
        )
        return self._get_cashpad(
            url=url, params=params, backend_id=backend_id, all_response=all_response
        )

    def _get_archive(
        self, installation_id, sequential_id, backend_id=False, all_response=False
    ):
        """
        Retourne le détail d'une archive
        https://cashpad.notion.site/Archive-content-44020e73cb1b437a925de8cb015979c1

        :param integer installation_id: id de la caisse
        :param integer sequential_id: id de la séquence

        :returns dict: réponse de l'API
        """
        url = "/api/salesdata/v2/" + installation_id + "/archive_content"
        params = {"sequential_id": sequential_id}
        return self._get_cashpad(
            url=url, params=params, backend_id=backend_id, all_response=all_response
        )

    def _get_sales_summary(
        self,
        installation_id,
        sequential_id,
        groups=None,
        backend_id=False,
        all_response=False,
    ):
        """
        Retourne le résumé des ventes pour une archive
        https://cashpad.notion.site/Sales-summary-8b5a6592cdb8466e9e7a19c0d752e7db

        :param integer installation_id: id de la caisse
        :param integer sequential_id: id de la séquence
        :param list groups: liste des groupes (period, consumptionmode, location)
        :param object backend_id: backend à interroger
        :param bool all_response: retourne la totalité de la réponse ou les données

        :returns dict: réponse de l'API
        """
        url = "/api/salesdata/v2/" + installation_id + "/sales_summary"
        params = {"sequential_id": sequential_id}
        if groups:
            params.update({"groups[]": groups})
        return self._get_cashpad(
            url=url, params=params, backend_id=backend_id, all_response=all_response
        )

    def _get_cash_containers(
        self,
        installation_id,
        sequential_id,
        backend_id=False,
        all_response=False,
    ):
        """
        Retourne le détail une archive
        https://cashpad.notion.site/Cash-containers-3ee592c4d21c40f18506df3fdf24dde5

        :param integer installation_id: id de la caisse
        :param integer sequential_id: id de la séquence
        :param object backend_id: backend à interroger
        :param bool all_response: retourne la totalité de la réponse ou les données

        :returns dict: réponse de l'API
        """
        url = "/api/salesdata/v2/" + installation_id + "/cashcontainers"
        params = {"sequential_id": sequential_id}
        return self._get_cashpad(
            url=url, params=params, backend_id=backend_id, all_response=all_response
        )

    # ------------------------------------------------------
    # Tools functions
    # ------------------------------------------------------
    def convert_datetime(self, date):
        return datetime.strptime(date, "%Y-%m-%dT%H:%M:%SZ")

    def convert_float(self, num):
        return num / 1000
